<?php

namespace Drupal\cms_content_sync\SyncCore;

use Drupal\Core\Url;

/**
 * Class Client
 * The client used by the Storage to connect to the Sync Core. You can imagine
 * this to be a remote database connection where the Storage talks to individual
 * tables / collections.
 *
 * @package Drupal\cms_content_sync\SyncCore
 */
class Client {

  /**
   * @var int MAX_ITEMS_PER_PAGE
   *    The maximum number of items that can be queried per page.
   */
  const MAX_ITEMS_PER_PAGE = 100;

  /**
   * @var \GuzzleHttp\Client
   *    The HTTP client used to execute requests.
   */
  protected $client;

  /**
   * @var string
   *    The base URL of the remote Sync Core. See Pool::$backend_url
   */
  protected $base_url;

  /**
   * @param string $base_url
   */
  public function __construct($base_url) {
    $this->base_url = $base_url;
    $this->client = \Drupal::httpClient();
  }

  /**
   * Execute a GET request.
   *
   * @param \Drupal\cms_content_sync\SyncCore\Query|string $query
   *
   * @return mixed
   *
   * @throws \Exception
   */
  public function get($query) {
    if (is_string($query)) {
      if (substr($query, 0, 7) == 'http://' || substr($query, 0, 8) == 'https://') {
        $url = preg_replace('@^https?://[^/]+/rest@', $this->base_url, $query);
      }
      else {
        $url = $this->base_url . $query;
      }
    }
    else {
      $url = $this->base_url . $query->getPath();

      $url = Url::fromUri($url, [
        'query' => $query->toArray(),
      ])->toUriString();
    }

    $response = $this->client->get($url);

    if ($response->getStatusCode() != 200) {
      throw new \Exception("Failed to get response from Sync Core (Code " . $response->getStatusCode() . "): " . $response->getBody());
    }

    $data = json_decode($response->getBody(), TRUE);

    return $data;
  }

}
